/* SPDX-License-Identifier: GPL-2.0 */
#undef TRACE_SYSTEM
#define TRACE_SYSTEM timestamp

#if !defined(_TRACE_TIMESTAMP_H) || defined(TRACE_HEADER_MULTI_READ)
#define _TRACE_TIMESTAMP_H

#include <linux/tracepoint.h>
#include <linux/fs.h>

TRACE_EVENT(ctime_floor_update,
	TP_PROTO(struct inode *inode,
		 ktime_t old,
		 ktime_t new,
		 ktime_t cur),

	TP_ARGS(inode, old, new, cur),

	TP_STRUCT__entry(
		__field(dev_t,				dev)
		__field(ino_t,				ino)
		__field(ktime_t,			old)
		__field(ktime_t,			new)
		__field(ktime_t,			cur)
	),

	TP_fast_assign(
		__entry->dev		= inode->i_sb->s_dev;
		__entry->ino		= inode->i_ino;
		__entry->old		= old;
		__entry->new		= new;
		__entry->cur		= cur;
	),

	TP_printk("ino=%d:%d:%lu old=%llu.%lu new=%llu.%lu cur=%llu.%lu swp=%c",
		MAJOR(__entry->dev), MINOR(__entry->dev), __entry->ino,
		ktime_to_timespec64(__entry->old).tv_sec,
		ktime_to_timespec64(__entry->old).tv_nsec,
		ktime_to_timespec64(__entry->new).tv_sec,
		ktime_to_timespec64(__entry->new).tv_nsec,
		ktime_to_timespec64(__entry->cur).tv_sec,
		ktime_to_timespec64(__entry->cur).tv_nsec,
		(__entry->old == __entry->cur) ? 'Y' : 'N'
	)
);

TRACE_EVENT(ctime_inode_update,
	TP_PROTO(struct inode *inode,
		 ktime_t old,
		 ktime_t new,
		 ktime_t cur),

	TP_ARGS(inode, old, new, cur),

	TP_STRUCT__entry(
		__field(dev_t,				dev)
		__field(ino_t,				ino)
		__field(ktime_t,			old)
		__field(ktime_t,			new)
		__field(ktime_t,			cur)
	),

	TP_fast_assign(
		__entry->dev		= inode->i_sb->s_dev;
		__entry->ino		= inode->i_ino;
		__entry->old		= old;
		__entry->new		= new;
		__entry->cur		= cur;
	),

	TP_printk("ino=%d:%d:%ld old=%llu.%ld new=%llu.%ld cur=%llu.%ld swp=%c",
		MAJOR(__entry->dev), MINOR(__entry->dev), __entry->ino,
		ktime_to_timespec64(__entry->old).tv_sec,
		ktime_to_timespec64(__entry->old).tv_nsec,
		ktime_to_timespec64(__entry->new).tv_sec,
		ktime_to_timespec64(__entry->new).tv_nsec,
		ktime_to_timespec64(__entry->cur).tv_sec,
		ktime_to_timespec64(__entry->cur).tv_nsec,
		(__entry->old == __entry->cur ? 'Y' : 'N')
	)
);

TRACE_EVENT(inode_needs_update_time,
	TP_PROTO(struct inode *inode,
		 struct timespec64 *now,
		 struct timespec64 *ctime,
		 int sync_it),

	TP_ARGS(inode, now, ctime, sync_it),

	TP_STRUCT__entry(
		__field(dev_t,			dev)
		__field(ino_t,			ino)
		__field(time64_t,		now_sec)
		__field(time64_t,		ctime_sec)
		__field(long,			now_nsec)
		__field(long,			ctime_nsec)
		__field(int,			sync_it)
	),

	TP_fast_assign(
		__entry->dev		= inode->i_sb->s_dev;
		__entry->ino		= inode->i_ino;
		__entry->sync_it	= sync_it;
		__entry->now_sec	= now->tv_sec;
		__entry->ctime_sec	= ctime->tv_sec;
		__entry->now_nsec	= now->tv_nsec;
		__entry->ctime_nsec	= ctime->tv_nsec;
		__entry->sync_it	= sync_it;
	),

	TP_printk("ino=%d:%d:%ld sync_it=%d now=%llu.%ld ctime=%llu.%lu",
		MAJOR(__entry->dev), MINOR(__entry->dev), __entry->ino,
		__entry->sync_it,
		__entry->now_sec, __entry->now_nsec,
		__entry->ctime_sec, __entry->ctime_nsec
	)
);

TRACE_EVENT(inode_set_ctime_to_ts,
	TP_PROTO(struct inode *inode,
		 struct timespec64 *ts),

	TP_ARGS(inode, ts),

	TP_STRUCT__entry(
		__field(dev_t,			dev)
		__field(ino_t,			ino)
		__field(time64_t,		ts_sec)
		__field(long,			ts_nsec)
	),

	TP_fast_assign(
		__entry->dev		= inode->i_sb->s_dev;
		__entry->ino		= inode->i_ino;
		__entry->ts_sec		= ts->tv_sec;
		__entry->ts_nsec	= ts->tv_nsec;
	),

	TP_printk("ino=%d:%d:%ld ts=%llu.%lu",
		MAJOR(__entry->dev), MINOR(__entry->dev), __entry->ino,
		__entry->ts_sec, __entry->ts_nsec
	)
);

TRACE_EVENT(fill_mg_cmtime,
	TP_PROTO(struct inode *inode,
		 ktime_t ctime),

	TP_ARGS(inode, ctime),

	TP_STRUCT__entry(
		__field(dev_t,			dev)
		__field(ino_t,			ino)
		__field(ktime_t,		ctime)
	),

	TP_fast_assign(
		__entry->dev		= inode->i_sb->s_dev;
		__entry->ino		= inode->i_ino;
		__entry->ctime		= ctime;
	),

	TP_printk("ino=%d:%d:%ld ctime=%llu.%lu",
		MAJOR(__entry->dev), MINOR(__entry->dev), __entry->ino,
		ktime_to_timespec64(__entry->ctime).tv_sec,
		ktime_to_timespec64(__entry->ctime).tv_nsec
	)
);
#endif /* _TRACE_TIMESTAMP_H */

/* This part must be outside protection */
#include <trace/define_trace.h>
